import java.util.concurrent.Executors;
import junit.framework.TestCase;
import org.infinispan.AdvancedCache;
import org.infinispan.affinity.KeyAffinityService;
import org.infinispan.affinity.KeyAffinityServiceFactory;
import org.infinispan.affinity.RndKeyGenerator;
import org.infinispan.config.Configuration;
import org.infinispan.config.GlobalConfiguration;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.manager.EmbeddedCacheManager;
import org.infinispan.test.TestingUtil;

public class KeyAffinityServiceTest extends TestCase {

    public static final int KEY_QUEUE_SIZE = 100;
    EmbeddedCacheManager manager1 = null, manager2 = null;
    AdvancedCache cache1 = null, cache2 = null;
    KeyAffinityService keyAffinityService1 = null, keyAffinityService2 = null;

    @Override
    protected void setUp() throws Exception {
        super.setUp();
        GlobalConfiguration gconf = new GlobalConfiguration();
        gconf.configureAsyncListenerExecutor().factory(org.infinispan.executors.DefaultExecutorFactory.class);
        gconf.configureAsyncListenerExecutor().addProperty("maxThreads", "15");
        gconf.configureAsyncListenerExecutor().addProperty("threadNamePrefix", "AsyncListenerThread");
        gconf.configureAsyncTransportExecutor().factory(org.infinispan.executors.DefaultExecutorFactory.class);
        gconf.configureAsyncTransportExecutor().addProperty("maxThreads", "25");
        gconf.configureAsyncTransportExecutor().addProperty("threadNamePrefix", "AsyncSerializationThread");
        gconf.configureEvictionScheduledExecutor().factory(org.infinispan.executors.DefaultScheduledExecutorFactory.class);
        gconf.configureEvictionScheduledExecutor().addProperty("threadNamePrefix", "EvictionThread");
        gconf.configureEvictionScheduledExecutor().factory(org.infinispan.executors.DefaultScheduledExecutorFactory.class);
        gconf.configureEvictionScheduledExecutor().addProperty("threadNamePrefix", "ReplicationQueueThread");
        gconf.configureGlobalJmxStatistics().enabled(Boolean.FALSE);
        gconf.configureGlobalJmxStatistics().allowDuplicateDomains(Boolean.FALSE);
        gconf.configureTransport().clusterName("infinispan-cluster");
        gconf.configureTransport().distributedSyncTimeout(50000l);
        gconf.configureTransport().transportClass(org.infinispan.remoting.transport.jgroups.JGroupsTransport.class);
        gconf.configureSerialization().marshallerClass(org.infinispan.marshall.VersionAwareMarshaller.class);
        gconf.configureSerialization().version("1.0");

        Configuration conf = new Configuration();
        conf.setInvocationBatchingEnabled(true);
        conf.configureClustering().configureHash().numOwners(1);
        conf.configureClustering().configureHash().rehashWait(60000l);
        conf.configureClustering().configureHash().rehashRpcTimeout(600000l);
        conf.configureClustering().mode(Configuration.CacheMode.DIST_SYNC);

        manager1 = new DefaultCacheManager(gconf);
        manager1.defineConfiguration("test", conf);
        cache1 = manager1.getCache("test").getAdvancedCache();
        keyAffinityService1 = KeyAffinityServiceFactory.newLocalKeyAffinityService(cache1, new RndKeyGenerator(), Executors.newSingleThreadExecutor(), KEY_QUEUE_SIZE);
        System.out.println("Address for manager1: " + manager1.getAddress());

        manager2 = new DefaultCacheManager(gconf);
        manager2.defineConfiguration("test", conf);
        cache2 = manager2.getCache("test").getAdvancedCache();
        keyAffinityService2 = KeyAffinityServiceFactory.newLocalKeyAffinityService(cache2, new RndKeyGenerator(), Executors.newSingleThreadExecutor(), KEY_QUEUE_SIZE);
        System.out.println("Address for manager2: " + manager2.getAddress());

        TestingUtil.blockUntilViewsReceived(60000, cache1, cache2);
    }

    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
        TestingUtil.killCacheManagers(manager1, manager2);
    }

    public void testKeyAffinityServiceFails() {
        System.out.println("Test keys for cache2.");
        for (int i = 0; i < KEY_QUEUE_SIZE; i++) {
            Object keyForAddress = keyAffinityService2.getKeyForAddress(manager2.getAddress());
            assertTrue(cache1.getDistributionManager().locate(keyForAddress).contains(manager2.getAddress()));
        }
        System.out.println("Test keys for cache1.");
        for (int i = 0; i < KEY_QUEUE_SIZE; i++) {
            Object keyForAddress = keyAffinityService1.getKeyForAddress(manager1.getAddress());
            assertTrue(cache1.getDistributionManager().locate(keyForAddress).contains(manager1.getAddress()));
        }
    }

    public void testKeyAffinityServiceSucceeds() {
        System.out.println("Clear keys.");
        for (int i = 0; i < KEY_QUEUE_SIZE; i++) {
            keyAffinityService1.getKeyForAddress(manager1.getAddress());
            keyAffinityService2.getKeyForAddress(manager2.getAddress());
        }
        System.out.println("Test keys for cache2.");
        for (int i = 0; i < KEY_QUEUE_SIZE; i++) {
            Object keyForAddress = keyAffinityService2.getKeyForAddress(manager2.getAddress());
            assertTrue(cache1.getDistributionManager().locate(keyForAddress).contains(manager2.getAddress()));
        }
        System.out.println("Test keys for cache1.");
        for (int i = 0; i < KEY_QUEUE_SIZE; i++) {
            Object keyForAddress = keyAffinityService1.getKeyForAddress(manager1.getAddress());
            assertTrue(cache1.getDistributionManager().locate(keyForAddress).contains(manager1.getAddress()));
        }
    }
}
